import requests
from bs4 import BeautifulSoup

def _get_infos(web_item, actual_version):
    version, __ = web_item.a.string.split('-')
    actual_version_on_web = version.replace(' ', '').removeprefix("Python")

    if actual_version != actual_version_on_web:
        download_list = web_item.ul.find_all("li")
        for link in download_list:
            #return only tarball links
            if link.a.string == "Gzipped source tarball":
                return {actual_version_on_web: link.a['href']}


def update_releases(dev_version, last_stable_version) -> dict:
    try:
        page = requests.get('https://www.python.org/downloads/source/')
        soup = BeautifulSoup(page.text, 'html.parser')
    except Exception:
        return
    
    lists = soup.find_all("div", class_="column")
    stables = lists[0].ul.find_all("li", recursive=False)
    #First ul.li in pre-releases column
    dev = lists[1].ul.li

    respost = {
        "new_dev": {},
        "new_stables": {}
    }

    new_dev = _get_infos(dev, dev_version)
    respost['new_dev'] = new_dev if new_dev else {}

    for web_item in stables:
        new_stable = _get_infos(web_item, last_stable_version)
        if new_stable:
            respost['new_stables'].update(new_stable)
        else:
            #If not found a new_stable, indicates the end
            break;

    return respost



from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='py-releases',
    packages=find_packages(include=['python_releases']),
    version='0.1.0',
    description='A package that get the avaliable python versions',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/kuticoski/py-releases",
    author='Gr Kuticoski',
    license='GPL-2.0',
    data_files=[('releases_infos', ['python_releases/releases_infos/infos.json'])],
    include_package_data=True,
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Operating System :: POSIX :: Linux"
    ],
    install_requires=[
        "beautifulsoup4==4.9.3",
        "requests==2.26.0"
    ]
)


# -*- coding: utf-8 -*-
from .infos import Infos

class Releases:
    def __init__(self) -> None:
        """Get required python versions.
    
        Attributes:
            infos (dict): infos about python versions
                data load from infos.json file
            dev (dict): version and download for the latest python pre-release
            last_stable (dict): version and download link of the last python stable
            all (dict): version and download link for all python releases
                the last item is the dev-version
        """
        self.infos = Infos().data

        self.last_stable = list(self.infos['stables'].items())[-1]
        self.dev = self.infos['dev_release']
        self.all = self._get_all_releases()
    
    def _get_all_releases(self) -> dict:
        all = self.infos['stables']
        all.update(self.dev)
        return all

    def search(self, version) -> str:
        """Search the download link from ´version´.

        Args:
            version (str): version to be searched
        Returns:
            str: Download link corresponding to version
        """
        try:
            return self.infos['stables'][version]
        except Exception:
            print("Version not found")



# -*- coding: utf-8 -*-
"""handler with infos."""

import json
import os

from datetime import datetime

from .scrapper import update_releases

class Infos:                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                
    def __init__(self, update=True) -> None:
        """Load the json file.
    
        Args:
            update (bool, opptinally): update the json file before loading data, if True.
        Attributes:
            data (dict): json loaded
        """
        self.data = self._load_json();
        if update:
            self.update(delay=0)
    
    def _load_json(self) -> dict:
        path = os.path.realpath(__file__)
        dir = os.path.dirname(path)

        file = open('{}/releases_infos/infos.json'.format(dir))
        json_data = json.load(file)
        file.close()

        return json_data
    
    def update(self, delay=3) -> None:
        """Update Json file and data attribute.
        
        Args:
            delay (int): indicates the minimum days for updating the json
                the value is compared with the last update date
        """
        now = datetime.now().date()
        last_update_date = datetime.strptime(
            self.data['last_update']['date'],
            '%Y-%m-%d'
        ).date()

        diff = now - last_update_date
        if diff.days >= delay:
            has_changed = False

            dev_version = list(self.data['dev_release'].keys())[0]
            last_stable = self.data['last_update']['stable_version']

            new_infos = update_releases(dev_version, last_stable)
            new_dev = new_infos['new_dev']
            new_stables = new_infos['new_stables']

            if new_dev:
                has_changed = True
                self.data['dev_release'] = new_dev
            
            if new_stables:
                has_changed = True

                #update stable version if exists new stable
                stable_version = list(new_infos['new_stables'].keys())
                stable_version = (
                    stable_version[0] if stable_version else last_stable
                )

                new_last_update = {
                    "date": now.strftime('%Y-%m-%d'),
                    "stable_version": stable_version
                }

                new_stables.update(self.data['stables'])
                new_stables = dict(sorted(
                    new_stables.items(),
                    key=lambda item: item[0]
                ))
                
                self.data['last_update'] = new_last_update
                self.data['stables'] = new_stables
            
            if has_changed:
                path = os.path.realpath(__file__)
                dir = os.path.dirname(path)
                file = '{}/releases_infos/infos.json'.format(dir)

                with open(file, 'w') as f:
                    json.dump(self.data, f, ensure_ascii=False, indent=4)
